/**
 * Dream Copyright (C) 2003-2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Matthieu Leclercq 
 * Contributor(s):
 */

package org.objectweb.dream.control.lifecycle;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.julia.control.content.Util;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This mixin provides a useable implementation of the
 * {@link org.objectweb.fractal.api.control.LifeCycleController#stopFc() }
 * method. This method stop every sub components providing a lifecycle
 * controller at once.
 * <p>
 * <p>
 * <b>Requirements </b>
 * <ul>
 * <li>This mixin must be used with the
 * {@link org.objectweb.fractal.julia.UseComponentMixin }mixin.</li>
 * <li>This mixin must be used with the {@link SimpleLifeCycleControllerMixin}
 * lifecycle controller implementation.</li>
 * <li>This mixin must be used with the {@link ThreadCounterMixin }lifecycle
 * controller extension.</li>
 * </ul>
 */
public abstract class FullLifeCycleMixin
{

  // ---------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    if (_this_weaveableFcState == 2)
    {
      _this_weaveableLCCLogger.log(BasicLevel.DEBUG, "Stop component");
      _this_stopFc(getFcLifeCycleControllers());
      _this_setFcState(false);
    }
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  /**
   * Returns the lifecycle control interface of sub components.
   * 
   * @return array of {@link LifeCycleCoordinator}of sub components.
   * @throws IllegalLifeCycleException if a problem occurs.
   */
  public LifeCycleCoordinator[] getFcLifeCycleControllers()
      throws IllegalLifeCycleException
  {
    Component thisComponent;
    try
    {
      thisComponent = (Component) _this_weaveableC.getFcInterface("component");
    }
    catch (NoSuchInterfaceException e)
    {
      throw new ChainedIllegalLifeCycleException(e, _this_weaveableC,
          "The DreamLifeCycleCoordinatorMixin requires components to provide "
              + "the Component interface");
    }
    List allSubComponents = Util.getAllSubComponents(thisComponent);
    List result = new ArrayList();
    Iterator iterator = allSubComponents.iterator();
    while (iterator.hasNext())
    {
      Component c = (Component) iterator.next();
      try
      {
        result.add((LifeCycleCoordinator) c
            .getFcInterface("lifecycle-controller"));
      }
      catch (Exception ignored)
      {
        // ignore
      }
    }
    return (LifeCycleCoordinator[]) result
        .toArray(new LifeCycleCoordinator[result.size()]);
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The <code>weaveableC</code> field required by this mixin. This field is
   * supposed to reference the {@link Component}interface of the component to
   * which this controller object belongs.
   */
  public Component _this_weaveableC;

  /**
   * the <code>weaveableFcState</code> field reqired by this mixin. This field
   * is supposed to reference the current state of the component lifecycle.
   * 
   * @see SimpleLifeCycleControllerMixin#weaveableFcState
   */
  public int       _this_weaveableFcState;

  /**
   * The <tt>weaveableLCCLogger</tt> field required by this mixin. This field
   * is supposed to reference the {@link Logger }of this controller.
   */
  public Logger    _this_weaveableLCCLogger;

  /**
   * The <tt>setFcState</tt> method required by this mixin. This method is
   * supposed to work as this
   * {@link org.objectweb.fractal.julia.control.lifecycle.BasicLifeCycleCoordinatorMixin#setFcState
   * setFcState} method.
   * 
   * @param started <tt>true</tt> to set the lifecycle state of the components
   *          to
   *          {@link org.objectweb.fractal.api.control.LifeCycleController#STARTED STARTED},
   *          or <tt>false</tt> to set this state to
   *          {@link org.objectweb.fractal.api.control.LifeCycleController#STOPPED STOPPED}.
   * @throws IllegalLifeCycleException if a problem occurs.
   */
  public abstract void _this_setFcState(boolean started)
      throws IllegalLifeCycleException;

  /**
   * the <code>stopFc</code> method required by this mixin. This method is
   * supposed to work a this
   * {@link org.objectweb.fractal.julia.control.lifecycle.BasicLifeCycleCoordinatorMixin#stopFc(LifeCycleCoordinator[])}
   * method.
   * 
   * @see org.objectweb.fractal.julia.control.lifecycle.BasicLifeCycleCoordinatorMixin#stopFc(LifeCycleCoordinator[])
   */
  public abstract void _this_stopFc(LifeCycleCoordinator[] components)
      throws IllegalLifeCycleException;
}