/**
 * Dream Copyright (C) 2003-2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Matthieu Leclercq 
 * Contributor(s):
 */

package org.objectweb.dream.control.lifecycle;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.InitializationContext;
import org.objectweb.fractal.julia.control.lifecycle.ChainedIllegalLifeCycleException;
import org.objectweb.fractal.julia.control.lifecycle.LifeCycleCoordinator;
import org.objectweb.fractal.julia.factory.ChainedInstantiationException;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Basic Dream life cycle controller. Component with this simple lifecycle
 * implementation can't be stopped alone (ie :{@link #stopFc() }throws a
 * {@link IllegalLifeCycleException }is the component is not already in the
 * stop state.
 * <p>
 * <p>
 * <b>Requirements </b>
 * <ul>
 * <li>This mixin must be used with the
 * {@link org.objectweb.fractal.julia.UseComponentMixin }mixin.</li>
 * <li>This mixin must be used with the
 * {@link org.objectweb.fractal.julia.control.lifecycle.BasicLifeCycleCoordinatorMixin}
 * lifecycle coordinator implementation.</li>
 * </ul>
 */
public abstract class SimpleLifeCycleControllerMixin
    implements
      LifeCycleCoordinator
{

  /**
   * The lifecycle state of the component. Zero means stopped, one means
   * stopping and two means started.
   */
  public int weaveableFcState;

  // ---------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.julia.Controller#initFcController(InitializationContext)
   */
  public void initFcController(final InitializationContext ic)
      throws InstantiationException
  {
    try
    {
      if (!(ic.getInterface("lifecycle-controller") instanceof LifeCycleCoordinator))
      {
        throw new Exception();
      }
    }
    catch (Exception e)
    {
      try
      {
        ic.getInterface("/lifecycle-coordinator");
      }
      catch (Exception f)
      {
        throw new ChainedInstantiationException(f, null,
            "The component must provide a LifeCycleCoordinator interface");
      }
    }

    _super_initFcController(ic);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
   */
  public String getFcState()
  {
    return weaveableFcState == 0 ? STOPPED : STARTED;
  }

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    if (weaveableFcState != 2)
    {
      _this_weaveableLCCLogger.log(BasicLevel.DEBUG, "Start component");
      _this_setFcState(true);
    }
  }

  /**
   * Throws a {@link IllegalLifeCycleException }if the component is in the
   * {@link org.objectweb.fractal.api.control.LifeCycleController#STARTED}
   * state.
   * 
   * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
   */
  public void stopFc() throws IllegalLifeCycleException
  {
    if (weaveableFcState == 2)
    {
      throw new ChainedIllegalLifeCycleException(null, _this_weaveableC,
          "This component can't be stopped alone");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleCoordinator interface
  // ---------------------------------------------------------------------------

  /**
   * @see LifeCycleCoordinator#setFcStarted()
   */
  public boolean setFcStarted() throws IllegalLifeCycleException
  {
    synchronized (this)
    {
      if (weaveableFcState == 2)
      {
        return false;
      }
      _this_weaveableLCCLogger.log(BasicLevel.DEBUG, "Component started");
      weaveableFcState = 2;
      return true;
    }
  }

  /**
   * Calls immediatly {@link LifeCycleCoordinator#fcInactivated }.
   * 
   * @see LifeCycleCoordinator#setFcStopping(LifeCycleCoordinator)
   */
  public void setFcStopping(LifeCycleCoordinator coordinator)
      throws IllegalLifeCycleException
  {
    synchronized (this)
    {
      _this_weaveableLCCLogger.log(BasicLevel.DEBUG, "Stopping component");
      weaveableFcState = 1;
      // immediatly inactive.
      isInactivated(coordinator);
    }
  }

  /**
   * @see LifeCycleCoordinator#setFcStopped()
   */
  public boolean setFcStopped() throws IllegalLifeCycleException
  {
    synchronized (this)
    {
      if (weaveableFcState == 0)
      {
        return false;
      }
      _this_weaveableLCCLogger.log(BasicLevel.DEBUG, "Component stopped");
      weaveableFcState = 0;
      return true;
    }
  }

  /**
   * Returns <code>true</code> if the component is inactive. This
   * implementation always returns <code>true</code>. Other mixins can
   * ovverride this method.
   * 
   * @return <code>true</code> if the component is inactive.
   */
  public boolean checkInactivity()
  {
    return true;
  }

  /**
   * If {@link #checkInactivity }returns <code>true</code>, calls
   * {@link LifeCycleCoordinator#fcInactivated }with the given
   * <code>coordinator</code> and returns <code>true</code>.
   * 
   * @param coordinator the coordinator passed to
   *          {@link LifeCycleCoordinator#fcInactivated }
   * @return the value of {@link #checkInactivity}
   */
  public boolean isInactivated(LifeCycleCoordinator coordinator)
  {
    if (checkInactivity())
    {
      _this_weaveableLCCLogger.log(BasicLevel.DEBUG, "Component inactivated");
      coordinator.fcInactivated(getFcCoordinator());
      return true;
    }
    return false;
  }

  // -------------------------------------------------------------------------
  // Utility methods
  // -------------------------------------------------------------------------

  /**
   * Returns the {@link LifeCycleCoordinator }interface of this component.
   * 
   * @return the {@link LifeCycleCoordinator }interface of the component to
   *         which this controller object belongs.
   */
  public LifeCycleCoordinator getFcCoordinator()
  {
    try
    {
      return (LifeCycleCoordinator) _this_weaveableC
          .getFcInterface("lifecycle-controller");
    }
    catch (Exception e)
    {
      try
      {
        return (LifeCycleCoordinator) _this_weaveableC
            .getFcInterface("/lifecycle-coordinator");
      }
      catch (NoSuchInterfaceException f)
      {
        throw new Error("Internal error"); // should not happen
      }
    }
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The <tt>weaveableC</tt> field required by this mixin. This field is
   * supposed to reference the {@link Component}interface of the component to
   * which this controller object belongs.
   */
  public Component _this_weaveableC;

  /**
   * The <tt>weaveableLCCLogger</tt> field required by this mixin. This field
   * is supposed to reference the {@link Logger }of this controller.
   */
  public Logger    _this_weaveableLCCLogger;

  /**
   * The <tt>setFcState</tt> method required by this mixin. This method is
   * supposed to work as this
   * {@link org.objectweb.fractal.julia.control.lifecycle.BasicLifeCycleCoordinatorMixin#setFcState
   * setFcState} method.
   * 
   * @param started <tt>true</tt> to set the lifecycle state of the components
   *          to {@link #STARTED STARTED}, or <tt>false</tt> to set this
   *          state to {@link #STOPPED STOPPED}.
   * @throws IllegalLifeCycleException if a problem occurs.
   */
  public abstract void _this_setFcState(boolean started)
      throws IllegalLifeCycleException;

  /**
   * The
   * {@link org.objectweb.fractal.julia.Controller#initFcController initFcController}
   * method overriden by this mixin.
   * 
   * @param ic information about the component to which this controller object
   *          belongs.
   * @throws InstantiationException if the initialization fails.
   */
  public abstract void _super_initFcController(InitializationContext ic)
      throws InstantiationException;
}