/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.dream.util.EmptyIterator;
import org.objectweb.dream.util.EmptyStringArray;
import org.objectweb.dream.util.Util;

/**
 * Abstract implementation of the <code>ExtensibleMessage</code> interface.
 * Methods managing sub messages are not implemented.
 */
public abstract class AbstractExtensibleMessage
    implements
      ExtensibleMessage,
      MessageType,
      MessageReferenceCounter,
      EmptyStringArray,
      Externalizable
{

  /** The id of the message manager that created this chunk. */
  protected transient short    messageManagerId;
  /** The number of reference to this message. */
  protected transient short    referenceCounter = 1;

  /** Contains (name, chunk object) pairs. */
  protected Map                chunks           = new HashMap();
  /** Contains (name, chunk type) pairs. */
  protected Map                chunkTypes       = new HashMap();

  /** The chunk names */
  protected transient String[] chunkNames       = null;

  // ---------------------------------------------------------------------------
  //  Implementation of the Message interface
  // ---------------------------------------------------------------------------

  /**
   * @see Message#getChunk(String)
   */
  public Object getChunk(String name)
  {
    return chunks.get(name);
  }

  /**
   * @see Message#getMessageType()
   */
  public MessageType getMessageType()
  {
    return this;
  }

  /**
   * @see Message#transfertChunkStates(Message)
   */
  public void transfertChunkStates(Message newInstance)
  {
    Iterator iter = chunks.entrySet().iterator();
    while (iter.hasNext())
    {
      Map.Entry e = (Map.Entry) iter.next();
      ((Chunk) e.getValue()).transfertState((Chunk) newInstance
          .getChunk((String) e.getKey()));
    }
  }

  /**
   * @see Message#getMessageManagerId()
   */
  public short getMessageManagerId()
  {
    return messageManagerId;
  }

  /**
   * @see Message#setMessageManagerId(short)
   */
  public void setMessageManagerId(short id)
  {
    messageManagerId = id;
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the Recyclable interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.pool.Recyclable#recycle()
   */
  public void recycle()
  {
    if (!chunks.isEmpty())
    {
      chunks.clear();
      chunkTypes.clear();
    }
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the ExtensibleMessage interface
  // ---------------------------------------------------------------------------

  /**
   * @see ExtensibleMessage#addChunk(String, ChunkType, Object)
   */
  public void addChunk(String name, ChunkType chkType, Object chunk)
      throws ChunkAlreadyExistsException
  {
    Object previousChunk = chunks.put(name, chunk);
    if (previousChunk != null)
    {
      chunks.put(name, previousChunk);
      throw new ChunkAlreadyExistsException();
    }
    chunkTypes.put(name, chkType);
    chunkNames = null;
  }

  /**
   * @see ExtensibleMessage#removeChunk(String)
   */
  public Object removeChunk(String name)
  {
    if (chunkTypes.remove(name) != null)
    {
      chunkNames = null;
    }
    return chunks.remove(name);
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the MessageType interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageType#getChunkNames()
   */
  public String[] getChunkNames()
  {
    if (chunkNames == null)
    {
      if (chunks.isEmpty())
      {
        chunkNames = EMPTY_STRING_ARRAY;
      }
      else
      {
        chunkNames = (String[]) chunks.keySet().toArray(EMPTY_STRING_ARRAY);
      }
    }
    return chunkNames;
  }

  /**
   * @see MessageType#getChunkNamesIterator()
   */
  public Iterator getChunkNamesIterator()
  {
    if (chunks.isEmpty())
    {
      return EmptyIterator.INSTANCE;
    }
    return chunks.keySet().iterator();
  }

  /**
   * @see MessageType#getChunkType(String)
   */
  public ChunkType getChunkType(String name)
  {
    return (ChunkType) chunkTypes.get(name);
  }

  /**
   * @see MessageType#isEmpty()
   */
  public boolean isEmpty()
  {
    return chunks.isEmpty();
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the MessageReferenceCounter interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageReferenceCounter#incrementReferenceCounter()
   */
  public void incrementReferenceCounter()
  {
    referenceCounter++;
  }

  /**
   * @see MessageReferenceCounter#decrementReferenceCounter()
   */
  public boolean decrementReferenceCounter()
  {
    synchronized (this)
    {
      referenceCounter--;
      return (referenceCounter == 0);
    }
  }

  // ---------------------------------------------------------------------------
  //  Serialisation methods
  // ---------------------------------------------------------------------------

  protected void readChunksState(ObjectInput in) throws IOException,
      ClassNotFoundException
  {
    int size = in.readInt();
    chunks = new HashMap(size);
    chunkTypes = new HashMap(size);
    for (int i = 0; i < size; i++)
    {
      String chunkName = in.readUTF();
      Object chunk = Util.readObject(in);
      Object chunkType = Util.readObject(in);
      chunks.put(chunkName, chunk);
      chunkTypes.put(chunkName, chunkType);
    }
  }

  protected void writeChunksState(ObjectOutput out) throws IOException
  {
    out.writeInt(chunks.size());
    Iterator iter = getChunkNamesIterator();
    while (iter.hasNext())
    {
      String chunkName = (String) iter.next();
      Object chunk = chunks.get(chunkName);
      Object chunkType = chunkTypes.get(chunkName);
      out.writeUTF(chunkName);
      Util.writeObject(out, chunk);
      Util.writeObject(out, chunkType);
    }
  }
}