/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq, Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.message;

import java.io.Serializable;
import java.util.Iterator;

import org.objectweb.dream.util.EmptyIterator;
import org.objectweb.dream.util.Error;

/**
 * Basic implementation of a non-extensible message. This class is intended to
 * be extended by specific non-extensible message implementations. It implements
 * the {@link org.objectweb.dream.message.Message}and
 * {@link org.objectweb.dream.message.MessageType}interfaces.
 */
public abstract class AbstractNonExtensibleMessage
    implements
      Message,
      MessageType,
      MessageReferenceCounter,
      Cloneable,
      Serializable
{

  /** The id of the message manager that created this message. */
  transient short messageManagerId;
  /** The number of reference to this message. */
  transient short referenceCounter = 0;

  // ---------------------------------------------------------------------------
  //  Implementation of the Message interface
  // ---------------------------------------------------------------------------

  /**
   * This class implements every chunk so it allways returns this. Warning : the
   * given chunk name is not checked
   * 
   * @see Message#getChunk(String)
   */
  public Object getChunk(String name)
  {
    return this;
  }

  /**
   * Allways returns {@link Message#EMPTY_MESSAGE_ARRAY}
   * 
   * @see Message#getSubMessages()
   */
  public Message[] getSubMessages()
  {
    return EMPTY_MESSAGE_ARRAY;
  }

  /**
   * @see Message#getSubMessageIterator()
   */
  public Iterator getSubMessageIterator()
  {
    return EmptyIterator.INSTANCE;
  }

  /**
   * @see Message#getMessageType()
   */
  public MessageType getMessageType()
  {
    return this;
  }

  /**
   * @see Message#getMessageManagerId()
   */
  public short getMessageManagerId()
  {
    return messageManagerId;
  }

  /**
   * @see Message#setMessageManagerId(short)
   */
  public void setMessageManagerId(short id)
  {
    messageManagerId = id;
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the MessageType interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageType#getSubMessageTypes()
   */
  public MessageType[] getSubMessageTypes()
  {
    Error.error("This method is not implemented", null);
    return EMPTY_MESSAGE_TYPE_ARRAY;
  }

  /**
   * @see MessageType#getSubMessageTypesIterator()
   */
  public Iterator getSubMessageTypesIterator()
  {
    Error.error("This method is not implemented", null);
    return EmptyIterator.INSTANCE;
  }

  /**
   * @see MessageType#isEmpty()
   */
  public boolean isEmpty()
  {
    Error.error("This method is not implemented", null);
    return false;
  }

  /**
   * @see MessageType#isSubTypeOf(MessageType)
   */
  public boolean isSubTypeOf(MessageType t)
  {
    Error.error("This method is not implemented", null);
    return false;
  }

  /**
   * @see MessageType#getChunkNames()
   */
  public String[] getChunkNames()
  {
    Error.error("This method is not implemented", null);
    return null;
  }

  /**
   * @see MessageType#getChunkNamesIterator()
   */
  public Iterator getChunkNamesIterator()
  {
    Error.error("This method is not implemented", null);
    return null;
  }

  /**
   * @see MessageType#getChunkType(String)
   */
  public ChunkType getChunkType(String name)
  {
    Error.error("This method is not implemented", null);
    return null;
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the MessageReferenceCounter interface
  // ---------------------------------------------------------------------------

  /**
   * @see MessageReferenceCounter#incrementReferenceCounter()
   */
  public void incrementReferenceCounter()
  {
    referenceCounter++;
  }

  /**
   * @see MessageReferenceCounter#decrementReferenceCounter()
   */
  public boolean decrementReferenceCounter()
  {
    synchronized (this)
    {
      referenceCounter--;
      return (referenceCounter == 0);
    }
  }
}