/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message;

import java.util.Iterator;

/**
 * A message type is a collection of pairs (name,
 * {@link org.objectweb.dream.message.ChunkType}) and sub message types.
 */
public interface MessageType
{

  /**
   * An empty <code>MessageType</code> array constant. This constant should be
   * used by classes requiring an empty <code>MessageType</code> array.
   */
  MessageType[] EMPTY_MESSAGE_TYPE_ARRAY = new MessageType[0];

  /**
   * Returns a chunk type of this message type from its name.
   * 
   * @param name the name of one of the chunk types returned by
   *          {@link #getChunkNames()}.
   * @return a chunk type of this message type from its name or
   *         <code>null</code> if no chunk with the specified name can be
   *         found.
   */
  ChunkType getChunkType(String name);

  /**
   * Returns an (eventually empty) array containing the chunk's names of
   * messages of this type.
   * 
   * @return an array containing the chunk's names of messages of this type.
   */
  String[] getChunkNames();

  /**
   * Returns an iterator over the chunk's names of messages of this type.
   * 
   * @return an iterator over the chunk's names of messages of this type.
   */
  Iterator getChunkNamesIterator();

  /**
   * Returns an (eventually empty) array containing the types of the sub
   * messages of messages of this type.
   * 
   * @return an array containing the types of the sub messages of messages of
   *         this type.
   */
  MessageType[] getSubMessageTypes();

  /**
   * Returns an iterator over the types of the sub messages of messages of this
   * type.
   * 
   * @return an iterator over the types of the sub messages of messages of this
   *         type.
   */
  Iterator getSubMessageTypesIterator();

  /**
   * Returns <code>true</code> if this message type is empty (ie : does not
   * contains any chunk types and any sub message types)
   * 
   * @return <code>true</code> if this message type is empty
   */
  boolean isEmpty();

  /**
   * Returns <code>true</code> if this message type if a sub type of the given
   * message type.
   * 
   * @param t a message type
   * @return <code>true</code> if this message type if a sub type of the given
   *         message type.
   */
  boolean isSubTypeOf(MessageType t);
}