/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package org.objectweb.dream.util;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.util.Fractal;

/**
 * This class defines utility methods.
 */
public final class Util
{

  private Util()
  {
  }

  /**
   * Retrieves the sub component with the specified name from the content of a
   * composite. Name must have the form "comp1/comp2/comp3".
   * 
   * @param cc the content controller of the composite.
   * @param name the name of the component to be retrieved.
   * @return the subcomponent with the specified name, <code>null</code> if no
   *         component with the specified name exists.
   */
  public static Component getComponentByName(ContentController cc, String name)
  {
    String[] compNames = name.split("/");
    ContentController contentController = cc;
    // The component to be returned
    Component c = null;
    for (int i = 0; i < compNames.length; i++)
    {
      c = null;
      Component[] comps = cc.getFcSubComponents();
      for (int j = 0; j < comps.length; j++)
      {
        Component component = comps[j];
        try
        {
          String tempName = Fractal.getNameController(component).getFcName();
          if (tempName.equals(compNames[i]))
          {
            c = component;
            cc = Fractal.getContentController(component);
            break;
          }
        }
        catch (NoSuchInterfaceException e)
        {
          // Do nothing
        }
      }
      if (c == null)
      {
        return null;
      }
    }
    return c;
  }

  /** A null object. Useful for Map values when only the key is meaningful. */
  public static final Object NULL_OBJECT = new Object();

  /**
   * Writes an array of bytes on a <code>DataOutput</code> object.
   * <p>
   * <i>Note: </i> if the array is <code>null</code>,<code>-1</code> is
   * written.
   * 
   * @param out the output on which to write.
   * @param array the arary to write.
   * @throws IOException if an error occurs.
   */
  public static void writeExternalByteArray(DataOutput out, byte[] array)
      throws IOException
  {
    if (array == null)
    {
      out.writeShort(-1);
    }
    else
    {
      short length = (short) array.length;
      out.writeShort(length);
      out.write(array);
    }
  }

  /**
   * Reads and returns an array of bytes from an <code>ObjectInput</code>
   * object.
   * 
   * @param in the input from which to read.
   * @return an array of byte.
   * @throws IOException if an error occurs.
   */
  public static byte[] readExternalByteArray(DataInput in) throws IOException
  {
    short length = in.readShort();
    if (length == -1)
    {
      return null;
    }
    else
    {
      byte[] toReturn = new byte[length];
      in.readFully(toReturn);
      return toReturn;
    }
  }

  /**
   * Writes an array of integer on a <code>DataOutput</code> object.
   * <p>
   * <i>Note: </i> if the array is <code>null</code>,<code>-1</code> is
   * written.
   * 
   * @param out the <code>ObjectOutput</code> on which to write.
   * @param array the arary to write.
   * @throws IOException if an error occurs.
   */
  public static void writeExternalIntArray(DataOutput out, int[] array)
      throws IOException
  {
    if (array == null)
    {
      out.writeShort(-1);
    }
    else
    {
      short length = (short) array.length;
      out.writeShort(length);
      for (int i = 0; i < length; i++)
      {
        out.writeInt(array[i]);
      }
    }
  }

  /**
   * Reads and returns an array of integer from a <code>DataInput</code>
   * object.
   * 
   * @param in the <code>ObjectInput</code> from which to read.
   * @return an array of integer.
   * @throws IOException if an error occurs.
   */
  public static int[] readExternalIntArray(DataInput in) throws IOException
  {
    short length = in.readShort();
    if (length == -1)
    {
      return null;
    }
    else
    {
      int[] toReturn = new int[length];
      for (int i = 0; i < length; i++)
      {
        toReturn[i] = in.readInt();
      }
      return toReturn;
    }
  }

  /**
   * Writes an array of long on a <code>DataOutput</code> object.
   * <p>
   * <i>Note: </i> if the array is <code>null</code>,<code>-1</code> is
   * written.
   * 
   * @param out the <code>ObjectOutput</code> on which to write.
   * @param array the arary to write.
   * @throws IOException if an error occurs.
   */
  public static void writeExternalLongArray(DataOutput out, long[] array)
      throws IOException
  {
    if (array == null)
    {
      out.writeShort(-1);
    }
    else
    {
      short length = (short) array.length;
      out.writeShort(length);
      for (int i = 0; i < length; i++)
      {
        out.writeLong(array[i]);
      }
    }
  }

  /**
   * Reads and returns an array of long from a <code>DataInput</code> object.
   * 
   * @param in the <code>ObjectInput</code> from which to read.
   * @return an array of integer.
   * @throws IOException if an error occurs.
   */
  public static long[] readExternalLongArray(DataInput in) throws IOException
  {
    short length = in.readShort();
    if (length == -1)
    {
      return null;
    }
    else
    {
      long[] toReturn = new long[length];
      for (int i = 0; i < length; i++)
      {
        toReturn[i] = in.readLong();
      }
      return toReturn;
    }
  }

  /**
   * Write the given object on the given {@link ObjectOutput}. If the given
   * output is an {@link ObjectOutputStream}, then use the
   * {@link ObjectOutputStream#writeUnshared(Object) }method, else use the
   * normal {@link ObjectOutput#writeObject(Object) }method.
   * 
   * @see ObjectOutput#writeObject(Object)
   */
  public static void writeObject(ObjectOutput output, Object o)
      throws IOException
  {
    if (output instanceof ObjectOutputStream)
    {
      ((ObjectOutputStream) output).writeUnshared(o);
    }
    else
    {
      output.writeObject(o);
    }
  }

  /**
   * Read the given object from the given {@link ObjectInput}. If the given
   * input is an {@link ObjectInputStream}, then use the
   * {@link ObjectInputStream#readUnshared() }method, else use the normal
   * {@link ObjectInput#readObject() }method.
   * 
   * @see ObjectInput#readObject()
   */
  public static Object readObject(ObjectInput input) throws IOException,
      ClassNotFoundException
  {
    if (input instanceof ObjectInputStream)
    {
      return ((ObjectInputStream) input).readUnshared();
    }
    else
    {
      return input.readObject();
    }
  }
}